"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const execa_1 = tslib_1.__importDefault(require("execa"));
exports.execa = execa_1.default;
const path_1 = tslib_1.__importDefault(require("path"));
const utils_1 = require("./utils");
function exec(commands, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (utils_1.Is.str(commands)) {
            return execa_1.default.shell(commands, options);
        }
        let rets = [];
        for (let cmd of commands) {
            rets.push(yield execa_1.default.shell(cmd, options));
        }
        return rets;
    });
}
exports.exec = exec;
exports.spawn = execa_1.default;
class ShellContext {
    constructor() {
        this._cwdStack = [process.cwd()];
        this._env = {};
        this.logCommand = false;
        this.sleep = utils_1.sleep;
    }
    /**
     * get current word directory
     */
    get cwd() {
        return this._cwdStack[this._cwdStack.length - 1];
    }
    get _logger() {
        let logger = require('./logger').logger;
        return logger;
    }
    /**
     * change work directory
     * @param dir
     */
    cd(dir) {
        this._cwdStack[this._cwdStack.length - 1] = path_1.default.resolve(this._cwdStack[this._cwdStack.length - 1], dir);
        return this;
    }
    /**
     * like pushd in shell
     * @param dir
     */
    pushd(dir) {
        this._cwdStack.push(path_1.default.resolve(this._cwdStack[this._cwdStack.length - 1], dir));
        return this;
    }
    /**
     * like popd in shell
     */
    popd() {
        this._cwdStack.pop();
        return this;
    }
    exec(commands, options) {
        this._logCmd(commands);
        let p = exec(commands, Object.assign({ cwd: this.cwd, env: Object.assign({}, process.env, this._env), stdio: 'inherit' }, options));
        // tslint:disable-next-line:no-floating-promises
        p.catch(err => {
            this._logger.error('Exec failed: ', commands);
            throw err;
        });
        return p;
    }
    /**
     * spawn file
     * @param file
     * @param args
     * @param options
     */
    spawn(file, args = [], options) {
        const command = file + ' ' + args.map(a => `"${a.replace(/"/g, '\\"')}"`).join(' ');
        this._logCmd(command);
        let p = exports.spawn(file, args, Object.assign({ cwd: this.cwd, env: Object.assign({}, process.env, this._env), stdio: 'inherit' }, options));
        // tslint:disable-next-line:no-floating-promises
        p.catch(err => {
            this._logger.error('Exec failed: ', command);
            throw err;
        });
        return p;
    }
    env(key, val) {
        if (arguments.length === 1) {
            return this._env[key];
        }
        if (utils_1.Is.defed(val)) {
            this._env[key] = val;
        }
        else {
            delete this._env[key];
        }
        return this;
    }
    /**
     * reset env to default
     */
    resetEnv() {
        this._env = {};
        return this;
    }
    _logCmd(cmd) {
        if (this.logCommand) {
            let env = Object.keys(this._env).map(k => `${k}=${this._env[k] || ''}`).join(' ');
            if (env) {
                env += ' ';
            }
            cmd = Array.isArray(cmd) ? cmd : [cmd];
            cmd.forEach(cmd => {
                this._logger.info(`$ ${env}${cmd}`);
            });
        }
    }
}
exports.ShellContext = ShellContext;
function shell(callback) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return callback(new ShellContext());
    });
}
exports.shell = shell;
//# sourceMappingURL=exec.js.map